/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <libintl.h>
#include <sys/types.h>

#include "ProcessTable.h"
#include "Mutex.h"
#include "Tools.h"
#include "sambascanner.h"

extern struct Options_Structure Options;
static struct Process *Processes = NULL;
static int NumberProcesses = 0;
static ProcessTableID CurrentID = 0;

/* After rewinding the ProcessTable it has to be completely traversed using
 * ProcessTable_Next. The ProcessTable gets locked upon rewinding and gets
 * unlocked after there is no "Next" element */
void ProcessTable_Rewind() {
	Lock_ProcessTable();
	CurrentID = -1;
#ifdef DEBUG
	/* Check ProcessTable consistency */
	{
		int Running = 0;
		int Invalid = 0;
		int i;
		for (i = 0; i < NumberProcesses; i++) {
			if (!Processes[i].Valid) Invalid++;
			if (Processes[i].Running) Running++;
		}
		message(5, _("Traversal of ProcessTable, consistency check: %d total in process list, %d running, %d invalid.\n"), NumberProcesses, Running, Invalid);
	}
#endif
}

ProcessTableID ProcessTable_Next() {
	do {
		CurrentID++;
		if (CurrentID == NumberProcesses) {
			/* Last element reached */
			Unlock_ProcessTable();
			return PROCESSTABLE_END;
		}
	} while (!Processes[CurrentID].Valid);

	/* Now we're standing on a valid element, return its ID */
	return CurrentID;
}

ProcessTableID ProcessTable_CreatePreliminary(double TermTime, double KillTime) {
	double CurrentTime = GetTime();
	
	Lock_ProcessTable();

	Processes = realloc(Processes, sizeof(struct Process) * (NumberProcesses + 1));
	NumberProcesses++;

	CurrentID = NumberProcesses - 1;
	Processes[CurrentID].Running = 1;
	Processes[CurrentID].Status = 0;
	Processes[CurrentID].Forwarding = 1;
	Processes[CurrentID].Signalling_State = 0;
	Processes[CurrentID].Valid = 0;
	Processes[CurrentID].TermTime = CurrentTime + (TermTime * Options.Timings);
	Processes[CurrentID].KillTime = CurrentTime + ((TermTime + KillTime) * Options.Timings);
	Processes[CurrentID].PID = 0;

	return CurrentID;
}

void ProcessTable_UpdateEntry_ThreadID(pthread_t ThreadID) {
	Processes[CurrentID].ThreadID = ThreadID;
}

void ProcessTable_FinishEntry(pid_t PID) {
	Processes[CurrentID].PID = PID;
	Processes[CurrentID].Valid = 1;
	Unlock_ProcessTable();
}


void ProcessTable_UpdateEntry_Times(ProcessTableID ProcID, double TermTime, double KillTime) {
	double CurrentTime = GetTime();
	Lock_ProcessTable();
	Processes[ProcID].TermTime = CurrentTime + (TermTime * Options.Timings);
	Processes[ProcID].KillTime = CurrentTime + ((TermTime + KillTime) * Options.Timings);
	Unlock_ProcessTable();
}

pid_t ProcessTable_GetCurrent_PID() {
	return Processes[CurrentID].PID;
}

int ProcessTable_GetCurrent_Running() {
	return Processes[CurrentID].Running;
}

double ProcessTable_GetCurrent_TermTime() {
	return Processes[CurrentID].TermTime;
}

double ProcessTable_GetCurrent_KillTime() {
	return Processes[CurrentID].KillTime;
}

int ProcessTable_GetCurrent_SignallingState() {
	return Processes[CurrentID].Signalling_State;
}

int ProcessTable_Get_Running(int ProcID) {
	int Running;	
	Lock_ProcessTable();
	Running = Processes[ProcID].Running;
	Unlock_ProcessTable();
	return Running;
}


pid_t ProcessTable_Get_PID(int ProcID) {
	pid_t PID;	
	Lock_ProcessTable();
	PID = Processes[ProcID].PID;
	Unlock_ProcessTable();
	return PID;
}

pthread_t ProcessTable_Get_TID(int ProcID) {
	pthread_t TID;	
	Lock_ProcessTable();
	TID = Processes[ProcID].ThreadID;
	Unlock_ProcessTable();
	return TID;
}

int ProcessTable_Get_Status(int ProcID) {
	int Status;	
	Lock_ProcessTable();
	Status = Processes[ProcID].Status;
	Unlock_ProcessTable();
	return Status;
}

int ProcessTable_Get_Forwarding(ProcessTableID ProcID) {
	int Fwd;
	Lock_ProcessTable();
	Fwd = Processes[ProcID].Forwarding;
	Unlock_ProcessTable();
	return Fwd;
}

void ProcessTable_SetCurrent_NotRunning() {
	Processes[CurrentID].Running = 0;
}

void ProcessTable_SetCurrent_Status(int Status) {
	Processes[CurrentID].Status = Status;
}

void ProcessTable_SetCurrent_SignallingState(int State) {
	Processes[CurrentID].Signalling_State = State;
}

void ProcessTable_Set_Forwarding_Closed(ProcessTableID ProcID) {
	Lock_ProcessTable();
	Processes[ProcID].Forwarding = 0;
	Unlock_ProcessTable();
}

